library window_with_count;

import 'dart:async';

class WindowWithCount<S, T> extends StreamEventTransformer {

  final int _count;
  final int _skip;

  const WindowWithCount(this._count, this._skip);

  Stream<T> bind(Stream<S> stream) {
    var controller = new StreamController();
    var broadcast  = stream.asBroadcastStream();
    var skip       = 0;
    var first      = true;
    broadcast.listen(
      (S data) {
        if (first) {
          var initial = new StreamController();
          first       = false;
          initial.add(data);
          controller.add(initial.stream);
          broadcast.take(this._count - 1).listen(
            (S data)         => initial.add(data),
            onError: (error) => initial.addError(error),
            onDone: ()       => initial.close()
          );
        }
        if (++skip >= this._skip) {
          controller.add(broadcast.take(this._count));
          skip = 0;
        }
      },
      onError: (error) {
        controller.addError(error);
      },
      onDone: () {
        controller.close();
      }
    );
    return controller.stream;
  }

}